from datetime import date, datetime
from decimal import Decimal
from typing import Optional
from uuid import UUID

from guarana.schemas.base import BaseModel


class OrderCall(BaseModel):
    subtotal: Decimal
    discount: Decimal
    total: Decimal
    created_at: datetime
    pricing_reference_date: date
    id_order: UUID
    id_commercial: int
    id_order_category: UUID
    id_customer: UUID
    id_coupon: Optional[UUID]



import json
from typing import Optional

import guarana
from guarana.schemas.customer import CustomerCall


class CustomerTracker(guarana.EventTracker):
    def __init__(self, segment_write_key: str):
        super().__init__(segment_write_key=segment_write_key)

    def identify_customer(
        self, customer_call: CustomerCall, anonymous_id: Optional[str] = None, **kwargs
    ):
        customer_json = customer_call.json(exclude_none=True, by_alias=True)
        customer_dict = json.loads(customer_json)
        customer_call_info = {**customer_dict, **kwargs}

        self.segment_client.identify(
            user_id=str(customer_call.user_id),
            anonymous_id=anonymous_id,
            traits=customer_call_info,
        )



from typing import Any, Optional

import segment.analytics as analytics
from loguru import logger


class SegmentClient:
    def __init__(self, write_key: str):
        self.client = analytics
        self.client.write_key = write_key
        self.client.on_error = self.on_error
    

    def identify(self, user_id: str, traits: dict, anonymous_id: Optional[str] = None) -> Any:
        try:
            self.client.identify(user_id=user_id, anonymous_id=anonymous_id, traits=traits)
            return { "success": True, "msg": "Identify method called successfully" }
        except Exception as e:
            return { "success": False, "msg": e }

    def track(
        self,
        event_name: str,
        properties: dict,
        user_id: str = None,
        anonymous_id: str = None,
    ) -> Any:   

        try:
            self.client.track(
                event=event_name,
                properties=properties,
                user_id=str(user_id),
                anonymous_id=str(anonymous_id),
            )
            return { "success": True, "msg": "Event logged successfully" }
        except Exception as e:
            return { "success": False, "msg": e }

    def on_error(error, items):
        logger.error(f'Error: {error} - {items != None: items | dict() }')



from datetime import datetime, time
from typing import Optional
from uuid import UUID

from pydantic import Field, root_validator

from guarana.schemas.base import BaseModel


class CustomerCall(BaseModel):
    email: str
    id_customer: UUID
    name: Optional[str] = Field(alias="firstname")
    street: Optional[str]
    number: Optional[str]
    complement: Optional[str]
    address: Optional[str]
    state: Optional[str]
    city: Optional[str]
    contact_name: Optional[str]
    cellphone: Optional[str] = Field(alias="phone")
    neighborhood: Optional[str] = Field(alias="bairro")
    payment_schedule_in_days: Optional[int]
    cep: Optional[str] = Field(alias="zip")
    cpf: Optional[str]
    cnpj: Optional[str]
    is_shopping_enabled: Optional[bool]
    id_customer_group: Optional[UUID]
    id_location: Optional[UUID]
    id_commercial: Optional[int]
    potential: Optional[str]
    corporate_name: Optional[str]
    tax_regime: Optional[str]
    ibge_city_code: Optional[str]
    nfe_validated: Optional[bool]
    nfe_validated_at: Optional[datetime]
    nfe_validated: Optional[bool]
    nfe_validated_at: Optional[datetime]
    crm_validated: Optional[bool]
    crm_validated_at: Optional[datetime]
    business_hours_open: Optional[time]
    business_hours_close: Optional[time]
    address_type: Optional[str]
    address_reference: Optional[str]
    cuisine_category: Optional[str]
    blocked_by_credit_threshold: Optional[bool]
    id_customer_origin: Optional[UUID]
    customer_type: Optional[str]
    is_blocked: Optional[bool]
    notes: Optional[str]
    id_delivery_category: Optional[UUID]

    @root_validator
    def build_address(cls, values) -> dict:
        if not values["street"]:
            return values

        address_string = values["street"]
        if values["number"]:
            address_string += f", {values['number']}"
        if values["complement"]:
            address_string += f" - {values['complement']}"

        values["address"] = address_string

        return values



__version__ = '0.1.0'
from loguru import logger

from .segment_client import SegmentClient


class EventTracker:
    def __init__(self, segment_write_key: str):
        self.segment_client = SegmentClient(segment_write_key)
        logger.info(f"Initialized SegmentClient")

    def identify_user(self, user_id: str, traits: dict):
        sucess, msg = self.segment_client.identify(user_id=user_id, traits=traits)
        if sucess:
            logger.success(msg)
        else:
            logger.warning(msg)



    def track(self, event_name: str, properties: dict, user_id: str = None, anonymous_id: str = None):
        sucess, msg = self.segment_client.track(
            event_name=event_name,
            properties=properties,
            user_id=user_id,
            anonymous_id=anonymous_id,
        )
        if sucess:
            logger.success(msg)
        else:
            logger.warning(msg)



from uuid import UUID

from pydantic import BaseModel as PydanticBaseModel


class BaseModel(PydanticBaseModel):
    user_id: UUID

    def __init__(self, **kwargs):
        kwargs["user_id"] = kwargs["id_customer"]
        super().__init__(**kwargs)

    class Config:
        json_encoders = {
            UUID: lambda x: str(x),
        }



import guarana
from guarana.schemas.events import OrderTrackEvents
from guarana.schemas.order import OrderCall


class OrderTracker(guarana.EventTracker):
    def __init__(self, segment_write_key: str):
        super().__init__(segment_write_key=segment_write_key)

    def track_order_placed(self, order_call: OrderCall, email: str, **kwargs):
        order_track_properties = order_call.dict(exclude_none=True, exclude_unset=True)
        identify_payload = {"email": email, "id_customer": str(order_call.id_customer)}

        self.segment_client.track(
            event_name=OrderTrackEvents.order_completed,
            user_id=str(order_call.user_id),
            properties={**order_track_properties, **kwargs, "email": email},
        )

        match str(order_call.id_order_category):
            case "055ebaa9-7583-4d47-9ce8-a5d772f7a91d":
                # Sampling order
                identify_payload |= {
                    "last_sampling_order_placed_ts": order_call.created_at,
                    "last_sampling_order_placed_id": str(order_call.id_order),
                    "last_sampling_order_placed_total": order_call.total,
                }
            case "44895221-7870-4aeb-b570-4ceb251e9663":
                # Standard order
                identify_payload |= {
                    "last_standard_order_placed_ts": order_call.created_at,
                    "last_standard_order_placed_id": str(order_call.id_order),
                    "last_standard_order_placed_total": order_call.total,
                }
            case "c3ebe8d1-fde0-4a62-af3e-2e0ce5642408":
                # Return order
                identify_payload |= {
                    "last_return_order_placed_ts": order_call.created_at,
                    "last_return_order_placed_id": str(order_call.id_order),
                    "last_return_order_placed_total": order_call.total,
                }

        identify_payload |= {
            "last_order_placed_ts": order_call.created_at,
            "last_order_placed_id": str(order_call.id_order),
            "last_order_placed_total": order_call.total,
        }

        if order_call.id_coupon:
            identify_payload |= {"last_coupon_used_id": str(order_call.id_coupon)}

        self.segment_client.identify(
            user_id=str(order_call.user_id),
            traits=identify_payload,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['guarana', 'guarana.schemas', 'guarana.trackers']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.6.0,<0.7.0',
 'pydantic>=1.10.2,<2.0.0',
 'segment-analytics-python>=2.2.0,<3.0.0']

setup_kwargs = {
    'name': 'guarana',
    'version': '2.3.1',
    'description': '',
    'long_description': 'None',
    'author': 'Rodrigo Godinho',
    'author_email': 'rodrigolegod@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
